#ifndef DTMSTRUCT_H
#define DTMSTRUCT_H

/***********************************************************************/
/* Some general constants */
#define	DTM_MAX_STRLEN	128


/***********************************************************************/
/* The poster generated by the module for P3D */
#define	DTM_MAX_LINES	200
#define	DTM_MAX_COLUMNS	200	/* Both sizes must be even, so that the
				   structure is aligned wrt. doubles */
#define DTM_P3D_POSTER_ASCII_FF	"DTM_ASCII_POSTER_file_format"
#define DTM_DEFAULT_GDHE_DTM_FILENAME "/tmp/gdheDtm.dtm"

typedef enum demCellState {DTM_CELL_EMPTY, DTM_CELL_FILLED} demCellState;

typedef struct DTM_P3D_POSTER {
  int		nbLines;
  int		nbCols;		/* Nb of lines and columns of the dtm */

  float		xOrigin;
  float		yOrigin;	/* Coordinates of the topleft corner of the
				   (0,0) pixel in the global frame */
  float		xScale;
  float		yScale;		/* Size of a cell in meters */


  float		zOrigin;	/* Elevation of the '0' value */
  float		zScale;		/* Scale to store the elevation */

  demCellState  state[DTM_MAX_LINES][DTM_MAX_COLUMNS]; /* To know if a cell is EMPTY or FILLED */

  float         zfloat[DTM_MAX_LINES][DTM_MAX_COLUMNS];  /* Altitude in float */
} DTM_P3D_POSTER;

/***********************************************************************/
/* Initialization params */

#define DTM_DEFAULT_X_SCALE	0.1
#define DTM_DEFAULT_Y_SCALE	0.1
#define DTM_DEFAULT_X_NB	400
#define DTM_DEFAULT_Y_NB	400
#define DTM_DEFAULT_X_ORIGIN	(- 10.0)
#define DTM_DEFAULT_Y_ORIGIN	(- 10.0)

/* File Format for saved dtm */
typedef enum DtmSlidingMode {
  DTM_FIXED_MODE,
  DTM_SLIDING_MODE
} DtmSlidingMode;

#define DTM_DEFAULT_SLIDING_MODE	(DTM_FIXED_MODE)

typedef struct DtmInitParams {
  float		xScale;
  float	      	yScale;		/* Scales of the DTM */
  int		xNb;
  int		yNb;		/* Nb pixels of the DTM */
  float		xOrigin;
  float      	yOrigin;	/* Position of the DTM origin in the global
				   frame */ 
  DtmSlidingMode	slidingMode;
  int			unused;
} DtmInitParams;


/***********************************************************************/
/* Connection params */

#define DTM_DEFAULT_POM_POSTER	   "pomPos"
#define DTM_DEFAULT_CAMERA_POSTER	"cameraImage0"
#define DTM_DEFAULT_STEREO_POSTER	"stereopixelThreeDImage"
#define DTM_DEFAULT_VELODYNE_POSTER	"velodyneThreeDImage"
#define DTM_DEFAULT_SICK_POSTER		"Sick"

typedef struct DtmConnectParams {
  char		pomPoster[DTM_MAX_STRLEN];
  int		stereoConnection;
  char		cameraPoster[DTM_MAX_STRLEN];
  char		stereoPoster[DTM_MAX_STRLEN];
  int		velodyneConnection;
  char		velodynePoster[DTM_MAX_STRLEN];
  int		sickConnection;
  char		sickPoster[DTM_MAX_STRLEN];
  int		unused;
} DtmConnectParams;


/***********************************************************************/
/* Save params */
#define	DTM_DEFAULT_BASENAME	"/tmp/"
#define DTM_DEFAULT_NUM		0
#define DTM_DEFAULT_SAVELOCALDTM 0

/* File Format for saved dtm */
typedef enum DtmFileFormat {
  DTM_ASCII,
  DTM_BINARY
} DtmFileFormat;

typedef struct DtmSaveParams {
    char	baseName[DTM_MAX_STRLEN];
    int		num;
    DtmFileFormat fileFormat;
	int saveLocalDtm;
} DtmSaveParams;

/***********************************************************************/
/* Geodesic origin params */
#define	DTM_DEFAULT_EASTING	0.0
#define	DTM_DEFAULT_NORTHING	0.0
#define	DTM_DEFAULT_ALTITUDE	0.0

typedef struct DtmGeodesicOrigin {
  double 	easting;
  double	northing;
  double	altitude;
} DtmGeodesicOrigin;

/***********************************************************************/
/* Fusion params */
typedef enum DtmVideo {
  DTM_USE_VIDEO,
  DTM_NO_VIDEO
} DtmVideo;
typedef enum DtmRefinement {
  DTM_USE_POSE_REFINEMENT,
  DTM_NO_POSE_REFINEMENT
} DtmRefinement;
typedef enum DtmReferenceFrame {
  DTM_REFERENCE_FRAME_GLOBAL,
  DTM_REFERENCE_FRAME_ROBOT
} DtmReferenceFrame;
typedef enum DtmStrategyKind {
	DTM_FIXED_TRESHOLD,
	DTM_MEAN_TRESHOLD
} DtmStrategyKind;

typedef struct DtmStrategy {
	DtmStrategyKind kind;
	double treshold;
} DtmStrategy;


#define DTM_DEFAULT_MAX_DIST	15.0
#define DTM_DEFAULT_VIDEO	DTM_NO_VIDEO
#define DTM_DEFAULT_REFINEMENT	DTM_NO_POSE_REFINEMENT
#define DTM_DEFAULT_REFINEMENT_RATE	5
#define DTM_DEFAULT_REFERENCE_FRAME DTM_REFERENCE_FRAME_GLOBAL
#define DTM_DEFAULT_STRATEGY_KIND	DTM_MEAN_TRESHOLD
#define DTM_DEFAULT_STRATEGY_TRESHOLD 3.0

typedef struct DtmFusionParams {
  double	    maxDistance;
  DtmVideo	    video;
  DtmRefinement	    poseRefinement;
  int		    refinementRate;
  DtmReferenceFrame referenceFrame;
  DtmStrategy strategy;
} DtmFusionParams;

/***********************************************************************/
/* Poster params */
#define	DTM_DEFAULT_POSTER_NB_LINES	150
#define	DTM_DEFAULT_POSTER_NB_COLS	150
#define	DTM_DEFAULT_Z_SCALE	0.001
#define	DTM_DEFAULT_Z_OFFSET	10

typedef struct DtmP3dPosterParams {
  int		nbLines;
  int		nbCols;
  float		zScale;
  float		zOffset;
} DtmPosterParams;

/***********************************************************************/
/* Display params */

#define DTM_DEFAULT_DPY_SERVER		"dala"
#define DTM_DEFAULT_DPY_FILE			"/tmp/tmpGdheDtm"
#define DTM_DEFAULT_DPY_NUMBER		0
#define DTM_DEFAULT_DPY_Z_OFFSET	1.0
#define DTM_DEFAULT_DPY_Z_SCALE		0.001
#define DTM_DEFAULT_DPY_COLOR		0
#define DTM_DEFAULT_DPY_CROP		1
#define DTM_DEFAULT_DPY_LIGHT		1

typedef struct DtmDisplayParams {
  char		serverName[DTM_MAX_STRLEN];
  char		displayFileName[DTM_MAX_STRLEN];
  int		displayFileNumber;
  double	zOffset;
  double	zScale;
  int		dpyColor;
  int		dpyCrop;
  int		dpyLight;
} DtmDisplayParams;

#endif
